import type { AxiosPromise } from 'axios'
import type {
    Patient,
    PatientReminder,
    THealthMatrix,
    HealthMatrixData,
    PatientAnalysis,
    PatientTreatmentCourse,
} from '@/entities'
import { medicalApi } from '@/shared'
import type {
    Medical,
    SetSurveyData,
    AddReminderData,
    TreatmentCourse,
    AddTreatmentCourseData,
    DeleteAppointmentData,
    DeleteTreatmentCourseFromPatientData,
    MedicalSurvey,
    ViewSurveyAnswersData,
    SurveyDetail,
    MedicalTest,
    AddOrUpdateOptimumData,
    MarkerOptimums,
    AddOrUpdateAnalysisData,
    TreatmentCourseData,
} from '../lib'

/**------------------ Surveys -------------------------- */
export const fetchSurveys = (
    search?: MedicalAPI.GET.FetchSurveys.Params,
): MedicalAPI.GET.FetchSurveys.Response =>
    medicalApi.get('survey', {
        params: { search },
    })

export const setSurveysToPatient = (
    data: MedicalAPI.PUT.SetSurveysToPatient.Params,
): MedicalAPI.PUT.SetSurveysToPatient.Response =>
    medicalApi.put(`customer/${data.customer_id}/survey`, {
        survey_ids: data.survey_ids,
    })

export const fetchSurveyQuestions = (
    params: MedicalAPI.GET.FetchQuestionsOfSurvey.Params,
): MedicalAPI.GET.FetchQuestionsOfSurvey.Response =>
    medicalApi.get(`survey/${params}`)

export const viewSurveyAnswers = (
    params: MedicalAPI.GET.FetchAnswersOfSurvey.Params,
): MedicalAPI.GET.FetchAnswersOfSurvey.Response =>
    medicalApi.get(
        `customer/${params.customer_id}/survey/${params.survey_attemp_id}`,
    )

/**------------------ Reminder -------------------------- */
export const addReminderToPatient = (
    data: MedicalAPI.POST.addReminderToPatient.Params,
): MedicalAPI.POST.addReminderToPatient.Response =>
    medicalApi.post('eventReminder', data)

/**------------------ Treatmen Course -------------------------- */
export const fetchTreatmentCourse = (
    search?: MedicalAPI.GET.FetchTreatmentCourse.Params,
): MedicalAPI.GET.FetchTreatmentCourse.Response =>
    medicalApi.get('treatmentCourse', {
        params: { search },
    })

export const addTreatmentCourseToPatient = (
    data: MedicalAPI.POST.AddTreatmentCourseToPatient.Params,
): MedicalAPI.POST.AddTreatmentCourseToPatient.Response =>
    medicalApi.post(`users/${data.user_id}/treatmentCourseUser`, data)

export const addFileToTreatment = (data: {
    user_id: number
    payload: FormData
}): MedicalAPI.POST.AddTreatmentCourseToPatient.Response =>
    medicalApi.post(`users/${data.user_id}/treatmentCourseUser`, data.payload)

export const fetchPatientTreatmentCourse = (
    data: MedicalAPI.GET.FetchPatientTreatmentCourse.Params,
): MedicalAPI.GET.FetchPatientTreatmentCourse.Response =>
    medicalApi.get(`users/${data}/treatmentCourseUser`)

export const deleteTreatmentCourseFromPatient = (
    data: MedicalAPI.DELETE.DeleteTreatmentCourseFromPatient.Params,
): MedicalAPI.DELETE.DeleteTreatmentCourseFromPatient.Response =>
    medicalApi.delete(
        `users/${data.user_id}/treatmentCourseUser/${data.treatment_course_id}`,
    )

export const editTreatmentCourse = (
    data: MedicalAPI.PUT.UpdateTreatmentCourse.Params,
): MedicalAPI.PUT.UpdateTreatmentCourse.Response =>
    medicalApi.put(
        `users/${data.user_id}/treatmentCourseUser/${data.treatment_course_id}`,
        data.payload,
    )

/**------------------ Appointment -------------------------- */

export const deleteAppointmentFromPatient = (
    data: MedicalAPI.DELETE.DeleteAppointmentFromPatient.Params,
): MedicalAPI.DELETE.DeleteAppointmentFromPatient.Response =>
    medicalApi.delete(`appointment/${data.appointment}`)

/**------------------ Medical Test -------------------------- */

export const fetchMedicalTests =
    (): MedicalAPI.GET.FetchAllMedicalTest.Response =>
        medicalApi.get('listMedicalTest')

export const updateCustomOptimum = (
    data: MedicalAPI.POST.UpdateCustomOptimum.Params,
): MedicalAPI.POST.UpdateCustomOptimum.Response =>
    medicalApi.post('optimalCustom', data)

export const addOrUpdateAnalysis = (
    data: MedicalAPI.POST.AddOrUpdateAnalysis.Params,
): MedicalAPI.POST.AddOrUpdateAnalysis.Response =>
    medicalApi.post(`users/${data.user_id}/analysis`, data)

/**------------------ Health Matrix -------------------------- */
export const updateHealthMatrixValue = (
    data: MedicalAPI.POST.UpdateHealthMatrixValue.Params,
): MedicalAPI.POST.UpdateHealthMatrixValue.Response =>
    medicalApi.post(`appointment/${data.appointment_id}/healthMatrix`, data)

export namespace MedicalAPI {
    export namespace GET {
        export namespace FetchSurveys {
            export type Params = string
            export type Response = AxiosPromise<{
                data: Medical['survey_list']
            }>
        }

        export namespace FetchTreatmentCourse {
            export type Params = string
            export type Response = AxiosPromise<{
                data: TreatmentCourse[]
            }>
        }

        export namespace FetchPatientTreatmentCourse {
            export type Params = Patient['id']
            export type Response = AxiosPromise<{
                data: TreatmentCourse[]
            }>
        }

        export namespace FetchAnswersOfSurvey {
            export type Params = ViewSurveyAnswersData
            export type Response = AxiosPromise<{
                data: SurveyDetail
            }>
        }

        export namespace FetchQuestionsOfSurvey {
            export type Params = MedicalSurvey['id']
            export type Response = AxiosPromise<{
                data: SurveyDetail
            }>
        }

        export namespace FetchAllMedicalTest {
            export type Response = AxiosPromise<{
                data: MedicalTest[]
            }>
        }
    }

    export namespace PUT {
        export namespace SetSurveysToPatient {
            export type Params = SetSurveyData
            export type Response = AxiosPromise<{
                data: any
            }>
        }

        export namespace UpdateTreatmentCourse {
            export type Params = TreatmentCourseData
            export type Response = AxiosPromise<{
                data: PatientTreatmentCourse
            }>
        }
    }

    export namespace POST {
        export namespace addReminderToPatient {
            export type Params = AddReminderData
            export type Response = AxiosPromise<{
                data: PatientReminder
            }>
        }

        export namespace AddTreatmentCourseToPatient {
            export type Params = AddTreatmentCourseData
            export type Response = AxiosPromise<{
                data: any
            }>
        }

        export namespace UpdateCustomOptimum {
            export type Params = AddOrUpdateOptimumData
            export type Response = AxiosPromise<{
                data: MarkerOptimums
            }>
        }

        export namespace AddOrUpdateAnalysis {
            export type Params = AddOrUpdateAnalysisData
            export type Response = AxiosPromise<{
                data: PatientAnalysis
            }>
        }
        export namespace UpdateHealthMatrixValue {
            export type Params = HealthMatrixData
            export type Response = AxiosPromise<{
                data: Maybe<THealthMatrix>
            }>
        }
    }

    export namespace DELETE {
        export namespace DeleteAppointmentFromPatient {
            export type Params = DeleteAppointmentData
            export type Response = AxiosPromise<{
                data: {
                    success: boolean
                    data: boolean
                    message: null | string
                }
            }>
        }

        export namespace DeleteTreatmentCourseFromPatient {
            export type Params = DeleteTreatmentCourseFromPatientData
            export type Response = AxiosPromise<{
                data: any
            }>
        }
    }
}
