import { defineStore, storeToRefs } from 'pinia'
import { computed, reactive, ref } from 'vue'
import { toast } from 'vue3-toastify'
import {
    addReminderToPatient,
    addTreatmentCourseToPatient,
    fetchMedicalTests,
    editTreatmentCourse,
    fetchPatientTreatmentCourse,
    fetchSurveyQuestions,
    fetchSurveys,
    fetchTreatmentCourse,
    setSurveysToPatient,
    setTestOptimums,
    updateCustomOptimum,
    viewSurveyAnswers,
    type PatientAnalysis,
    addFileToTreatment,
} from '@/entities'
import { usePatientStore } from '@/entities'
import { Stores, declension } from '@/shared'
import type {
    Medical,
    SetSurveyData,
    AddReminderData,
    AddTreatmentCourseData,
    ViewSurveyAnswersData,
    BaseAnalysisOptimumValues,
    MarkerOptimums,
    AddOrUpdateOptimumData,
    TreatmentCourseData,
} from '../lib'

type MedicalState = BaseState<Maybe<Medical>>

export const useMedicalStore = defineStore(Stores.MEDICAL, () => {
    /**
     * State
     */

    const state: MedicalState = reactive({
        data: {
            survey_list: [],
            treatment_courses: [],
            patient_treatment: [],
            survey: null,
            medical_test: [],
        },
        loading: false,
    })

    const currOptimums = ref<BaseAnalysisOptimumValues[]>([])
    const { analysisResults, infoForMedicalTest } = storeToRefs(
        usePatientStore(),
    )

    /**
     * Getters
     */

    const surveyList = computed(() => {
        return state.data?.survey_list?.map(x => ({
            ...x,
            questions_count: declension(Number(x.questions_count), [
                'вопрос',
                'вопроса',
                'вопросов',
            ]),
        }))
    })

    const treatmentCourses = computed(
        () =>
            state.data?.treatment_courses?.map(x => ({
                ...x,
                duration: declension(x.duration, ['день', 'дня', 'дней']),
            })),
    )

    const patientTreatments = computed(() => {
        return (
            state.data?.patient_treatment?.map(x => ({
                id: x.id,
                title: x.title,
            })) || []
        )
    })

    const currSurvey = computed(() => state.data?.survey)

    const medicalTestList = computed(() => {
        if (state.data?.medical_test && state.data.medical_test?.length) {
            const newList: any[] = []

            state.data.medical_test.forEach((x, idx) => {
                const date: Set<string> = new Set()
                const markers: any[] = []

                x.markers?.forEach(el => {
                    const result: PatientAnalysis[] = []
                    let optimums: MarkerOptimums | undefined
                    let interval: string = ''

                    if (currOptimums.value[idx]['type'] == 'custom') {
                        optimums = el.optimums_custom?.find(x => {
                            return (
                                String(
                                    currOptimums.value[idx]['value'],
                                ).includes(x.age) &&
                                currOptimums.value[idx].sex == x.sex
                            )
                        })
                    } else {
                        optimums = el.optimums?.find(x => {
                            return (
                                String(
                                    currOptimums.value[idx]['value'],
                                ).includes(x.age) &&
                                currOptimums.value[idx].sex == x.sex
                            )
                        })
                    }

                    analysisResults.value?.forEach(a => {
                        if (a.marker_id === el.id) {
                            let analysisState: string = ''

                            if (a.result) {
                                if (
                                    optimums?.min &&
                                    optimums?.max &&
                                    !Number.isNaN(optimums?.min) &&
                                    !Number.isNaN(optimums?.max)
                                ) {
                                    if (optimums.min > a.result) {
                                        analysisState = 'down'
                                    } else if (
                                        optimums.min <= a.result &&
                                        a.result <= optimums.max
                                    ) {
                                        analysisState = 'normal'
                                    } else {
                                        analysisState = 'up'
                                    }

                                    interval = `${
                                        optimums?.min + '-' + optimums?.max
                                    }`
                                } else if (optimums?.min) {
                                    if (optimums.min > a.result) {
                                        analysisState = 'down'
                                    } else if (optimums.min == a.result) {
                                        analysisState = 'normal'
                                    } else {
                                        analysisState = 'up'
                                    }
                                    interval = `${optimums?.min}`
                                }
                            }

                            date.add(a.date)

                            result.push({
                                ...a,
                                state: analysisState,
                            })
                        }
                    })

                    markers.push({
                        id: el.id,
                        name: el.name,
                        unit: el.unit,
                        list_medical_test_id: el.list_medical_test_id,
                        tip_min: el.tip_min,
                        tip_max: el.tip_max,
                        notice: el.notice,
                        result,
                        optimums: {
                            ...optimums,
                            interval,
                        },
                    })
                })

                newList.push({
                    id: x.id,
                    title: x.title,
                    unit: x.unit,
                    analyze_date: [...date],
                    markers,
                })
            })

            return newList
        } else {
            return []
        }
    })

    const hasCustomGetters = (testId: number, markerId: number, age: string) =>
        computed(() => {
            const medicalTest = state.data?.medical_test.find(
                x => x.id == testId,
            )
            const testMarker = medicalTest?.markers.find(x => x.id == markerId)

            return testMarker?.optimums_custom.find(x => x.age == age)
        })

    const updateCurrOptimumsVal = (optimum: {
        value: string
        sex: string
        type: 'base' | 'custom'
        test_id: number
    }) => {
        currOptimums.value.forEach((el, idx) => {
            if (el.test_id == optimum.test_id) {
                currOptimums.value[idx] = {
                    ...optimum,
                    sex:
                        optimum.sex == 'children'
                            ? infoForMedicalTest.value.sex
                            : optimum.sex,
                }
                return
            }
        })
    }
    /**
     * Actions
     */

    const setSurveyList = async (search: string) => {
        try {
            state.loading = true
            const { data } = await fetchSurveys(search)

            if (data.data && state.data?.survey_list) {
                state.data.survey_list = data.data
            }
        } catch (e) {
            console.log(e)
        }
        state.loading = false
    }

    const setTreatmentCourseList = async (search: string) => {
        try {
            state.loading = true
            const { data } = await fetchTreatmentCourse(search)
            if (data.data?.length && state.data?.treatment_courses) {
                state.data.treatment_courses = data.data
            }
        } catch (e: any) {
            console.log('e ->', e)
        }
        state.loading = false
    }

    const setPatientTreatmentCourseList = async (payload: number) => {
        try {
            state.loading = true
            const { data } = await fetchPatientTreatmentCourse(payload)

            if (data.data?.length && state.data?.patient_treatment) {
                state.data.patient_treatment = data.data
            }
        } catch (e: any) {
            console.log('e ->', e)
        }
        state.loading = false
    }

    const addDestinationToPatient = async (payload: AddTreatmentCourseData) => {
        try {
            state.loading = true
            const { data } = await addTreatmentCourseToPatient(payload)
            if (data.data) {
                usePatientStore().setDataToState(data.data)
            }

            toast.success('Успешно сохранено!')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
        }
        state.loading = false
    }

    const addFileToTreatmentCourse = async (payload: any) => {
        try {
            state.loading = true
            const { data } = await addFileToTreatment(payload)
            if (data.data) {
                usePatientStore().setDataToState(data.data)
            }

            toast.success('Успешно сохранено!')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
        }
        state.loading = false
    }

    const addSurveyToPatient = async (payload: SetSurveyData) => {
        try {
            state.loading = true
            const { data } = await setSurveysToPatient(payload)
            usePatientStore().setDataToState(data.data)
            toast.success('Успешно сохранено!')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
        }
        state.loading = false
    }

    const addReminder = async (payload: AddReminderData) => {
        try {
            state.loading = true
            const { data } = await addReminderToPatient(payload)

            usePatientStore().setReminderToState(data.data)
            toast.success('Успешно сохранено!')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
        }
        state.loading = false
    }

    const viewSurvey = async (survey_id: number) => {
        state.loading = true
        try {
            state.loading = true
            const { data } = await fetchSurveyQuestions(survey_id)
            if (data.data && state.data) {
                state.data.survey = data.data
            }
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
        }
        state.loading = false
    }

    const viewSurveyResult = async (payload: ViewSurveyAnswersData) => {
        try {
            state.loading = true
            const { data } = await viewSurveyAnswers(payload)
            if (data.data && state.data) {
                state.data.survey = data.data

                state.data.survey['questions'] = data.data.questions.map(x => ({
                    ...x,
                    options: x.options.map(y => ({
                        ...y,
                        model: y.is_selected ? y.id : '',
                    })),
                }))
            }
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
        }
        state.loading = false
    }

    const setMedicalTest = async () => {
        try {
            state.loading = true
            const { data } = await fetchMedicalTests()
            if (data.data && data.data?.length && state.data?.medical_test) {
                currOptimums.value = setTestOptimums(
                    data.data,
                    infoForMedicalTest.value,
                )
                state.data.medical_test = data.data
            }
        } catch (e: any) {
            console.log('e ->', e)
        }

        state.loading = true
    }

    /**
     * только для кастомных оптимумов
     * @param {AddOrUpdateOptimumData} payload
     */
    const addOrUpdateCustomOptimum = async (
        payload: AddOrUpdateOptimumData,
    ) => {
        try {
            const { data } = await updateCustomOptimum(payload)
            toast.success('Успешно сохранено!')

            if (data.data && data.data?.marker_id) {
                const testIdx = state.data?.medical_test.length || 0
                let markerIdx, optimumIdx, i, j
                testLoop: for (i = 0; i < testIdx; i++) {
                    if (
                        state.data?.medical_test[i]['id'] ==
                        payload.list_medical_test_id
                    ) {
                        markerIdx =
                            state.data?.medical_test[i]?.markers?.length || 0
                        for (j = 0; j < markerIdx; j++) {
                            if (
                                state.data.medical_test[i]['markers'][j][
                                    'id'
                                ] == payload.marker_id
                            ) {
                                optimumIdx =
                                    state.data.medical_test[i]['markers'][j]?.[
                                        'optimums_custom'
                                    ]?.length || 0
                                let hasOptimum = false
                                for (let k = 0; k < optimumIdx; k++) {
                                    if (
                                        state.data.medical_test[i]['markers'][
                                            j
                                        ]?.['optimums_custom'][k]['sex'] ==
                                            payload.sex &&
                                        state.data.medical_test[i]['markers'][
                                            j
                                        ]?.['optimums_custom'][k]['age'] ==
                                            payload.age
                                    ) {
                                        state.data.medical_test[i]['markers'][
                                            j
                                        ]['optimums_custom'][k] = {
                                            ...data.data,
                                        }
                                        hasOptimum = true
                                    }
                                }

                                if (!hasOptimum) {
                                    state.data.medical_test[i]['markers'][j][
                                        'optimums_custom'
                                    ].push({
                                        ...data.data,
                                    })

                                    break testLoop
                                }
                            }
                        }
                    }
                }
            }
        } catch (e: any) {
            console.log('e ->', e)
        }
    }

    const updateTreatmentCourse = async (payload: TreatmentCourseData) => {
        try {
            const { data } = await editTreatmentCourse(payload)

            if (data.data) {
                usePatientStore().setTreatmentDataToState(data.data)
            }
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
        }
    }

    return {
        state,
        surveyList,
        currSurvey,
        treatmentCourses,
        patientTreatments,
        medicalTestList,
        currOptimums,
        hasCustomGetters,
        updateCurrOptimumsVal,
        setSurveyList,
        addSurveyToPatient,
        addReminder,
        setTreatmentCourseList,
        addDestinationToPatient,
        setPatientTreatmentCourseList,
        viewSurvey,
        viewSurveyResult,
        setMedicalTest,
        addOrUpdateCustomOptimum,
        updateTreatmentCourse,
        addFileToTreatmentCourse,
    }
})
