import { type AxiosPromise } from 'axios'
import { baseApi, medicalApi } from '@/shared'
import type {
    Appointments,
    EditAppointmentData,
    EditPatientData,
    Patient,
    PatientAnalysis,
    PatientMediaFile,
} from '../lib'

/**------------------ Patient -------------------------- */
export const fetchPatients = async ({
    search,
    page,
    per_page,
}: PatientAPI.GET.FetchPatients.Params): PatientAPI.GET.FetchPatients.Response => {
    const response = await baseApi.get(
        `customer?search=${search}&page=${page}&perPage=${per_page}`,
    )

    return response.data
}

export const fetchPatient = async (
    data: PatientAPI.GET.FetchPatient.Params,
): PatientAPI.GET.FetchPatient.Response => {
    const response = await baseApi.get(`customer/${data}`)

    return response.data
}

export const deletePatient = (data: PatientAPI.DELETE.DeletePatient.Params) =>
    new Promise(resolve => {
        setTimeout(() => {
            console.log('delete: ', data)

            resolve(data)
        }, 1000)
    })

export const editPatient = (
    data: PatientAPI.PUT.EditPatient.Params,
): PatientAPI.PUT.EditPatient.Response =>
    baseApi.put(`customer/${data.id}`, data)

/**------------------ Patient Appointment -------------------------- */
export const editPatientAppointment = (
    data: PatientAPI.PUT.EditAppointment.Params,
): PatientAPI.PUT.EditAppointment.Response =>
    medicalApi.put(`appointment/${data.id}`, data)

export const addAppointmentToPatient = (
    data: PatientAPI.POST.AddAppointment.Params,
): PatientAPI.POST.AddAppointment.Response =>
    medicalApi.post('appointment', data)

/**------------------ Set Avatar of Customer -------------------------- */
export const setPatientAvatar = (
    data: PatientAPI.POST.SetAvatar.Params,
): PatientAPI.POST.SetAvatar.Response => baseApi.post('user/setAvatar', data)

/**------------------ Media Files -------------------------- */
export const setFilesToPatient = (
    data: PatientAPI.POST.SetFiles.Params,
): PatientAPI.POST.SetFiles.Response =>
    medicalApi.post(`customer/${data.customer_id}/file`, data.files)

export const deleteMediaFile = async (
    data: PatientAPI.DELETE.DeleteFile.Params,
) => medicalApi.delete(`file/${data}`)

/**------------------ Medical Tests -------------------------- */

export const fetchCustomerMedicalTest = (
    customer_id: PatientAPI.GET.FetchCustomerMedicalTests.Params,
): PatientAPI.GET.FetchCustomerMedicalTests.Response =>
    medicalApi.get(`users/${customer_id}/analysis`)

/**
 * Module PatientAPI
 * Interfaces:
 *   EditPatientParams
 *   EditAppointmentParams
 */
interface EditPatientParams extends EditPatientData {
    id: number
}

interface EditAppointmentParams extends EditAppointmentData {
    id: Appointments['id']
    user_id: Appointments['user_id']
}
export namespace PatientAPI {
    export namespace GET {
        export namespace FetchPatients {
            export type Params = {
                page: number
                per_page: number
                search: string
            }
            export type Response = AxiosPromise<PaginationData<Patient[]>>
        }

        export namespace FetchPatient {
            export type Params = Patient['id']
            export type Response = AxiosPromise<Patient>

            // export type Response = AxiosResponse<{
            //     data: Patient[]
            // }>
        }

        export namespace FetchCustomerMedicalTests {
            export type Params = Patient['id']
            export type Response = AxiosPromise<{
                data: PatientAnalysis[]
            }>
        }
    }

    export namespace DELETE {
        export namespace DeletePatient {
            export type Params = Patient['id']
        }

        export namespace DeleteFile {
            export type Params = PatientMediaFile['id']
        }
    }

    export namespace PUT {
        export namespace EditPatient {
            export type Params = EditPatientParams
            export type Response = AxiosPromise<{
                data: Patient
            }>
        }

        export namespace EditAppointment {
            export type Params = EditAppointmentParams
            export type Response = AxiosPromise<{
                data: Appointments
            }>
        }
    }

    export namespace POST {
        export namespace AddAppointment {
            export type Params = {
                user_id: Appointments['user_id']
            }
            export type Response = AxiosPromise<{
                data: {
                    user_id: number
                    updated_at: string
                    created_at: string
                    id: number
                }
            }>
        }

        export namespace SetAvatar {
            export type Params = FormData
            export type Response = AxiosPromise<{
                data: Patient
            }>
        }

        export namespace SetFiles {
            export type Params = {
                customer_id: Patient['id']
                files: FormData
            }
            export type Response = AxiosPromise<{
                data: Patient
            }>
        }
    }
}
