import { defineStore } from 'pinia'
import { computed, reactive, ref } from 'vue'
import { toast } from 'vue3-toastify'
import {
    editPatient,
    fetchPatient,
    editPatientAppointment,
    addAppointmentToPatient,
    setPatientAvatar,
    setFilesToPatient,
    deleteMediaFile,
    deleteAppointmentFromPatient,
    deleteTreatmentCourseFromPatient,
    updateHealthMatrixValue,
    type DeleteTreatmentCourseFromPatientData,
    fetchCustomerMedicalTest,
    addOrUpdateAnalysis,
    type AddOrUpdateAnalysisData,
    type HealthMatrixData,
} from '@/entities'
import {
    Stores,
    dateToAge,
    declension,
    formattingDateForClient,
    getTimeFromDate,
    prettifyDate,
} from '@/shared'
import {
    PatientStep,
    type Appointments,
    type EditAppointmentData,
    type EditPatientData,
    type Patient,
    type PatientReminder,
    type PatientMediaFile,
    type PatientAnalysis,
    type PatientTreatmentCourse,
} from '../../lib'

type PatientState = BaseState<Maybe<Patient>>
export const APPOINTMENTSITEMS: {
    key:
        | 'taking_medication'
        | 'taking_bud'
        | 'physical_activity'
        | 'stress'
        | 'sleep'
        | 'bad_habits'
        | 'complaint'
    name: any
}[] = [
    {
        name: 'Прием медикаментов',
        key: 'taking_medication',
    },
    {
        name: 'Прием бадов',
        key: 'taking_bud',
    },
    {
        name: 'Физическая активность',
        key: 'physical_activity',
    },
    {
        name: 'Стресс',
        key: 'stress',
    },
    {
        name: 'Сон',
        key: 'sleep',
    },
    {
        name: 'Вредные привычки',
        key: 'bad_habits',
    },
    {
        name: 'Жалобы',
        key: 'complaint',
    },
]

export const usePatientStore = defineStore(Stores.PATIENT, () => {
    /**-------- State -------------- */

    const state = reactive<PatientState>({
        loading: false,
        data: null,
    })

    const analysisResults = ref<PatientAnalysis[]>([])
    const idxAppointment = ref(0)
    const patientStep = ref<PatientStep>(PatientStep.MAIN)

    /**-------- Action -------------- */
    const setCurrentPatient = async (id: Patient['id']) => {
        try {
            state.loading = true

            const { data } = await fetchPatient(id)
            state.data = data
            idxAppointment.value = data?.appointments?.[0]?.id || 0
        } catch (e) {
            console.log(e)
        } finally {
            state.loading = false
        }
    }

    const resetCurrentPatient = () => {
        state.data = null
    }

    const onEditPatient = async (payload: EditPatientData) => {
        try {
            const { data } = await editPatient({
                ...payload,
                id: Number(state.data?.id),
            })
            state.data = data.data
            toast.success('Изменения сохранены !')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
        state.loading = false
    }

    const onEditAppointment = async (payload: EditAppointmentData) => {
        try {
            const { data } = await editPatientAppointment({
                ...payload,
                id: Number(currAppointment.value?.id),
                user_id: Number(currAppointment.value?.user_id),
            })
            const appointment = data.data
            let appointmentID = -1
            state.data?.appointments.forEach((el, i) => {
                if (el.id == appointment.id) {
                    appointmentID = i
                }
            })

            if (typeof appointmentID == 'number' && appointmentID > -1) {
                APPOINTMENTSITEMS.forEach(elem => {
                    if (state.data?.appointments[appointmentID]) {
                        state.data.appointments[appointmentID][elem.key] =
                            appointment[elem.key]
                    }
                })
            }
            toast.success('Изменения сохранены!')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
    }

    const onCreateAppointment = async (payload: {
        user_id: Appointments['user_id']
    }) => {
        try {
            await addAppointmentToPatient(payload)
            await setCurrentPatient(payload.user_id)
            toast.success('Успешно добавлено новый прием!')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
    }

    const onUpdateAvatar = async (payload: FormData) => {
        try {
            const { data } = await setPatientAvatar(payload)
            if (data.data) {
                state.data = data.data
            }
            toast.success('Изменения сохранены')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
    }

    const onUploadFiles = async (payload: FormData) => {
        try {
            const { data } = await setFilesToPatient({
                customer_id: Number(state.data?.id),
                files: payload,
            })

            if (data.data) {
                state.data = data.data
            }
            toast.success('Файлы успешно сохранены')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
    }

    const onDeleteMediaFile = async (payload: {
        id: PatientMediaFile['id']
        type: 'patient' | 'treatmentCourse'
    }) => {
        try {
            state.loading = true

            await deleteMediaFile(payload.id)

            if (payload.type == 'patient' && state.data?.media) {
                const filteredMedia = state.data.media.filter(
                    file => file.id !== payload.id,
                )

                state.data.media = filteredMedia
            }

            if (payload.type == 'treatmentCourse') {
                state.data?.appointments?.forEach((app, idx) => {
                    if (
                        app.id == currAppointment.value?.id &&
                        state.data?.appointments[idx]?.treatment_course_user
                    ) {
                        const media =
                            state.data.appointments[
                                idx
                            ].treatment_course_user?.media.filter(
                                x => x.id != payload.id,
                            ) || []

                        state.data.appointments[idx].treatment_course_user = {
                            analysis_and_research:
                                currAppointment.value.treatment_course_user
                                    ?.analysis_and_research || '',
                            appointment_id:
                                currAppointment.value.treatment_course_user
                                    ?.appointment_id || 0,
                            buds:
                                currAppointment.value.treatment_course_user
                                    ?.buds || '[]',
                            comment:
                                currAppointment.value.treatment_course_user
                                    ?.comment || '',
                            created_at:
                                currAppointment.value.treatment_course_user
                                    ?.created_at || '',
                            duration:
                                currAppointment.value.treatment_course_user
                                    ?.duration || 0,
                            enabled:
                                currAppointment.value.treatment_course_user
                                    ?.enabled || 0,
                            id:
                                currAppointment.value.treatment_course_user
                                    ?.id || 1,
                            media: media,
                            medication:
                                currAppointment.value.treatment_course_user
                                    ?.medication || '[]',
                            nutrition:
                                currAppointment.value.treatment_course_user
                                    ?.nutrition || '',
                            performer_id:
                                currAppointment.value.treatment_course_user
                                    ?.performer_id || 1,
                            title:
                                currAppointment.value.treatment_course_user
                                    ?.title || '',
                            updated_at:
                                currAppointment.value.treatment_course_user
                                    ?.updated_at || '',
                            user_id:
                                currAppointment.value.treatment_course_user
                                    ?.user_id || 1,
                        }
                    }
                })
            }

            toast.success('Файл удален')
        } catch (e) {
            console.log(e)

            toast.error('Произошла ошибка')
        } finally {
            state.loading = false
        }
    }

    const onDeleteAppointment = async (id: Appointments['id']) => {
        try {
            await deleteAppointmentFromPatient({ appointment: id })

            if (state.data?.appointments) {
                state.data.appointments = state.data?.appointments.filter(
                    x => x.id != id,
                )
            }
            if (idxAppointment.value == id) {
                idxAppointment.value = state.data?.appointments?.[0]?.id || 0
            }
            toast.success('Прием удален')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
    }

    const onDeleteTreatmentCourse = async (
        payload: DeleteTreatmentCourseFromPatientData,
    ) => {
        try {
            await deleteTreatmentCourseFromPatient(payload)
            state.data?.appointments?.forEach((app, idx) => {
                if (
                    app.id == currAppointment.value?.id &&
                    state.data?.appointments[idx]?.treatment_course_user
                ) {
                    state.data.appointments[idx].treatment_course_user =
                        undefined
                }
            })

            toast.success('Назначения удален')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
    }

    const setPatientAnalysis = async () => {
        try {
            if (state.data?.id) {
                const { data } = await fetchCustomerMedicalTest(state.data.id)

                if (data.data && Array.isArray(data.data)) {
                    analysisResults.value = data.data
                }
            }
        } catch (e: any) {
            console.log('e -> ', e)
        }
    }

    const updatePatientAnalysisResult = async (
        payload: AddOrUpdateAnalysisData,
    ) => {
        try {
            const { data } = await addOrUpdateAnalysis(payload)
            const lenAnalysis = analysisResults.value.length
            let isChanged = false

            for (let i = 0; i < lenAnalysis; i++) {
                if (
                    analysisResults.value[i]['marker_id'] ==
                        payload.marker_id &&
                    analysisResults.value[i]['date'] == payload.date
                ) {
                    analysisResults.value[i]['result'] = Number(
                        data?.data?.result || payload.result,
                    )
                    isChanged = true
                    break
                }
            }

            if (!isChanged) {
                // If analysisResult has not been changed, then this is a new analysis
                analysisResults.value.push(data.data)
            }
        } catch (e: any) {
            console.log('e -> ', e)
        }
    }

    const updateAnalysisDate = async (payload: AddOrUpdateAnalysisData) => {
        try {
            const { data } = await addOrUpdateAnalysis(payload)
            const lenAnalysis = analysisResults.value.length

            for (let i = 0; i < lenAnalysis; i++) {
                if (
                    analysisResults.value[i]['marker_id'] ==
                        payload.marker_id &&
                    analysisResults.value[i]['id'] == data.data.id
                ) {
                    analysisResults.value[i] = {
                        ...data.data,
                    }
                    break
                }
            }
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
    }

    const onUpdateHealthMatrix = async (payload: HealthMatrixData) => {
        try {
            const { data } = await updateHealthMatrixValue(payload)
            if (data.data?.appointment_id) {
                for (let i = 0; i < appointmentLen.value; i++) {
                    if (
                        state.data?.appointments[i]?.health_matrix
                            ?.appointment_id == data.data.appointment_id
                    ) {
                        state.data.appointments[i].health_matrix = {
                            ...data.data,
                        }
                    }
                }
            }
            toast.success('Изменения сохранены!')
        } catch (e: any) {
            toast.error(`Что-то не так! ${e?.message || ''}`)
            console.log('e -> ', e)
        }
    }

    const setDataToState = (data: Patient) => {
        state.data = data
    }

    const setReminderToState = (data: PatientReminder) => {
        state.data?.event_reminder_customer.push(data)
    }

    const setTreatmentDataToState = (data: PatientTreatmentCourse) => {
        state.data?.appointments?.forEach((app, idx) => {
            if (
                app.id == currAppointment.value?.id &&
                state.data?.appointments[idx]
            ) {
                state.data.appointments[idx].treatment_course_user = data
            }
        })
    }

    /**-------- Getters -------------- */
    const appointmentLen = computed(() => state.data?.appointments?.length || 0)
    const patientInfo = computed(() => ({
        sex: state.data?.sex || '',
        name: state.data?.name || '',
        city: state.data?.city || '',
        avatar: state.data?.avatar || '',
        marital: state.data?.marital || '',
        contact: state.data?.contact || '',
        birthdate: state.data?.birthdate || '',
        profession: state.data?.profession || '',
        children_count: state.data?.children_count || '',
        birthday: prettifyDate(state.data?.birthdate) || '',
        age:
            declension(dateToAge(state.data?.birthdate || '0'), [
                'год',
                'года',
                'лет',
            ]) || '',
        asking: state.data?.asking || '',
        anamnesis: state.data?.anamnesis || '',
    }))

    const analyzes = computed(
        () =>
            state.data?.medical_test?.map(x => ({
                ...x,
                created_at: prettifyDate(x.created_at),
            })) || [],
    )

    const survey = computed(
        () =>
            state.data?.survey_attempts?.map(x => ({
                id: x.id,
                title: x.survey?.title,
                total: x.survey?.questions_count || 0,
                answers: x.answers_count || 0,
                percent: x.percent,
            })),
    )

    const files = computed(() => state.data?.files)

    const media = computed(() => state.data?.media)

    const matrixHealth = computed(() => state.data?.health_matrix)

    const reminders = computed(
        () =>
            state.data?.event_reminder_customer?.map(x => ({
                id: x.id,
                date: String(
                    formattingDateForClient(x.datetime, 'short'),
                ).replace(/\./g, ''),
                time: getTimeFromDate(x.datetime),
                type: x.type,
                name: x.text,
            })),
    )

    const appointments = computed(
        () =>
            state.data?.appointments?.map(x => ({
                id: x.id,
                name: prettifyDate(x.created_at) || '',
            })),
    )

    const currAppointment = computed(
        () => state.data?.appointments?.find(x => x.id == idxAppointment.value),
    )

    const infoForMedicalTest = computed(
        (): {
            sex: string
            age: number
        } => ({
            sex: state.data?.sex || '',
            age: dateToAge(state.data?.birthdate || '0'),
        }),
    )

    const treatmentCourse = computed((): PatientTreatmentCourse | null => {
        return currAppointment.value?.treatment_course_user || null
    })

    return {
        state,
        media,
        files,
        survey,
        analyzes,
        reminders,
        patientInfo,
        patientStep,
        matrixHealth,
        appointments,
        idxAppointment,
        currAppointment,
        analysisResults,
        infoForMedicalTest,
        treatmentCourse,
        setCurrentPatient,
        resetCurrentPatient,
        onEditPatient,
        onEditAppointment,
        onCreateAppointment,
        setDataToState,
        setReminderToState,
        setTreatmentDataToState,
        onUpdateAvatar,
        onUploadFiles,
        onDeleteMediaFile,
        onDeleteAppointment,
        onDeleteTreatmentCourse,
        setPatientAnalysis,
        updatePatientAnalysisResult,
        updateAnalysisDate,
        onUpdateHealthMatrix,
    }
})
