import { defineStore } from 'pinia'
import { ref } from 'vue'
import { useRouter } from 'vue-router'
import { toast } from 'vue3-toastify'
import { type AuthData, Routes, Stores, useJWT } from '@/shared'
import { login, loginYandex, logout, registration } from '../../api'

enum MedicType {
    DOCTOR = 'doctor',
    NUTRITIONIST = 'nutritionist',
}

export type EducationData = {
    permission: boolean
    passport: Maybe<File>
    diploma: Maybe<File>
    medicType: Maybe<MedicType>
}

export type RegistrationData = AuthData & EducationData

export const useAuthStore = defineStore(Stores.AUTH, () => {
    const router = useRouter()
    const loading = ref(false)

    const { removeJWT, setJWT } = useJWT()

    const onLogin = async (data: AuthData) => {
        try {
            loading.value = true
            const res = await login(data)

            setJWT(res.data.data.access_token)

            await router.push({ name: Routes.INDEX })
        } catch (e: any) {
            console.log(e.response.data.message)

            if (e.response.data.message === 'Invalid login credentials') {
                toast.error('Неверные логин или пароль')
            }
        } finally {
            loading.value = false
        }
    }

    const onYandex = async () => {
        try {
            loading.value = true;
            const res = await loginYandex();
            const redirect_url = res.data
            console.log(res.data);
            window.location.href = redirect_url;
        } catch (e: any) {
            console.log(e.response.data.message)
        } finally {
            loading.value = false;
        }
    }

    const onRegistration = async (data: NonNullable<RegistrationData>) => {
        try {
            loading.value = true
            const res = await registration(data)

            setJWT(res.data.data.access_token)

            await router.push({ name: Routes.INDEX })
        } catch (e: any) {
            console.log(e.response.data.message)

            toast.error('Такой пользователь уже зарегестрирован')
        } finally {
            loading.value = false
        }
    }

    const onLogout = async () => {
        try {
            loading.value = true
            await logout()

            removeJWT()

            await router.push({ name: Routes.LOGIN })
        } catch (e: any) {
            console.log(e.response.data.message)
        } finally {
            loading.value = false
        }
    }

    return {
        onLogin,
        onYandex,
        onRegistration,
        onLogout,
        loading,
    }
})
